#!/bin/bash
#
# Do a backup or a restore of a local folder on an Amazon S3 bucket, using
# the s3sync.rb script (http://s3sync.net/).
# Modified on $Date: 2008-12-03 18:36:10 +0100 (Mer, 03 Dic 2008) $
# SVN $Revision: 48 $
#
# If launched with the name "s3backup.sh" it does the backup, while if launched
# with the name "s3restore.sh" it does the restore.
#
# Copyright (C) - 2008
# Alessandro Morgantini - gpz500@technologist.com
# http://gpz500.wordpress.com/2008/11/30/amazon-s3-e-mac-os-x-ma-non-solo/
#
# Released under the terms of GPL2 (http://www.gnu.org/licenses/gpl-2.0.html).

################################################################################
# Script's configuration

# Amazon S3 access ID info.
# If not specified here, s3sync.rb will search these info in the following
# locations:
#
#   $S3CONF/s3config.yml
#   $HOME/.s3conf/s3config.yml
#   /etc/s3conf/s3config.yml
#
# Read the s3sync.rb readme file at
# http://s3.amazonaws.com/ServEdge_pub/s3sync/README.txt.
#AWS_ACCESS_KEY_ID=1111111111111111111111111
#AWS_SECRET_ACCESS_KEY=2222222222222222222222222

# HTTP proxy data, if any.
# If not specified here, s3sync.rb will search these info in the following
# locations:
#
#   $S3CONF/s3config.yml
#   $HOME/.s3conf/s3config.yml
#   /etc/s3conf/s3config.yml
#
# Read the s3sync.rb readme file at
# http://s3.amazonaws.com/ServEdge_pub/s3sync/README.txt. 
#HTTP_PROXY_HOST=my.proxy.server
#HTTP_PROXY_PORT=8080
#HTTP_PROXY_USER=myusername
#HTTP_PROXY_PASSWORD=secret

# Name of Amazon S3 bucket to use
BUCKET=gpz500.wordpress.com

# s3sync.rb prefix to use. It defaults to the hostname, if not defined
PREFIX=mac-di-ale

# Absolute path of s3sync.rb script
S3SYNC=/Users/gpz500/s3sync/s3sync.rb

# End of script's configuration
################################################################################

# Export defined environment variables
if [ "x$AWS_ACCESS_KEY_ID" != "x" ]; then
	export AWS_ACCESS_KEY_ID
fi
if [ "x$AWS_SECRET_ACCESS_KEY" != "x" ]; then
	export AWS_SECRET_ACCESS_KEY
fi
if [ "x$HTTP_PROXY_HOST" != "x" ]; then
	export HTTP_PROXY_HOST
fi
if [ "x$HTTP_PROXY_PORT" != "x" ]; then
	export HTTP_PROXY_PORT
fi
if [ "x$HTTP_PROXY_USER" != "x" ]; then
	export HTTP_PROXY_USER
fi
if [ "x$HTTP_PROXY_PASSWORD" != "x" ]; then
	export HTTP_PROXY_PASSWORD
fi

# Init working variables
SRC_PATH=""
DEST_PATH=""
DELETE=""
NOMD5=""
DRYRUN=""
MAKEDIRS=""
VERBOSE=""
COMMAND_NAME=$(basename $0)
if [ "x$PREFIX" == "x" ]; then
	PREFIX=$(hostname | cut -f 1 -d .)
fi
	

# Check the correctness of name
if [ "$COMMAND_NAME" != s3backup.sh -a "$COMMAND_NAME" != s3restore.sh ]; then
	echo "Error: you must invoke this program as \"s3backup.sh\" or \"s3restore.sh\""
	exit 3
fi

# Print the usage instructions
usage ()
{
	echo "Usage: $COMMAND_NAME [options] /source/folder [/destination/folder]"
	echo ""
	echo "Options:   --delete, -d    Delete files in destination"
	echo "                           unless they are in the source."
	echo "           --dryrun, -n    Don't execute anything, actually."
	echo "           --make-dirs     Create intermediate folders"
	echo "                           when necessary."
	echo "           --no-md5        Disable the MD5 checksum on"
	echo "                           local files."
	echo "           --verbose, -v   Increase verbosity."
	echo ""
	echo "Warning: use only absolute paths"
}

# Discover if the supplied path is in a HFS+ volume
ishfs ()
{
	local RES=1
	if [ -x /Developer/Tools/SplitForks ]; then
		local TMPFILE="$1/s3backup$$.tmp"
		echo "This is a test file" >"$TMPFILE"
		/Developer/Tools/SplitForks "$TMPFILE" >/dev/null 2>&1
		RES=$?
		rm "$TMPFILE"
	fi
	
	return $RES
}

# Print the supplied string, only if VERBOSE is defined
writeout ()
{
	if [ -n "$VERBOSE" ]; then
		if [ "x$1" == "x-n" ]; then
			shift
			echo -n "$*"
		else
			echo "$*"
		fi
	fi
}

# Print info
if [ -z "$1" ]; then
	usage
	exit 0
fi

# Parse the parameters
while [ -n "$1" ]; do
	case $1 in
		--delete|-d)
			DELETE=--delete
			shift
			;;
		--dryrun|-n)
			DRYRUN=--dryrun
			shift
			;;
		--make-dirs)
			MAKEDIRS=--make-dirs
			shift
			;;
		--no-md5)
			NOMD5=--no-md5
			shift
			;;
		--verbose|-v)
			VERBOSE=--verbose
			shift
			;;
		-*)
			echo "Error: option \"$1\" is unknown"
			usage
			exit 2
			;;
		*)
			if [ "x$SRC_PATH" == "x" ]; then
				SRC_PATH=$1
			elif [ "x$DEST_PATH" == "x" ]; then
				DEST_PATH=$1
			else
				echo "Error: too many parameters"
				usage
				exit 3
			fi
			shift
			;;
	esac
done

# Check if the origin is correctly defined
if [ "${SRC_PATH:0:1}x" != "/x" ]; then
	echo "Error: you must supply an absolute path as source"
	usage
	exit 1
fi
if [ ! -d "$SRC_PATH" -a "$COMMAND_NAME" == s3backup.sh ]; then
	echo "Error: you must supply an existing folder as source"
	usage
	exit 4
fi

# If the destination path is not defined, set it equal to the source path.
# If there is a trailing slash, use the same name for destination also,
# if not, use the parent folder
if [ "x$DEST_PATH" == "x" ]; then
	if [[ $SRC_PATH == */ ]]; then
		DEST_PATH=$SRC_PATH
	else
		DEST_PATH=$(dirname "$SRC_PATH")
	fi
fi

# Check if the destination is correctly defined
if [ "${DEST_PATH:0:1}x" != "/x" ]; then
	echo "Error: you must supply an absolute path as destination"
	usage
	exit 5
fi
if [ ! -d "$DEST_PATH" -a "$COMMAND_NAME" == s3restore.sh ]; then
	echo "Error: you must supply an existing folder as destination"
	usage
	exit 6
fi


# Output some info
writeout "  I'm using the following settings"

# Run s3sync.rb
if [ "$COMMAND_NAME" == s3backup.sh ]; then
	writeout "  Source:      $SRC_PATH"
	writeout "  Destination: $BUCKET:$PREFIX$DEST_PATH"
	writeout ""
	writeout "  Starting s3sync... "
	ishfs "$SRC_PATH" && find "$SRC_PATH" -type f -xdev -exec /Developer/Tools/SplitForks {} \;
	ruby $S3SYNC -r $DELETE $NOMD5 $VERBOSE $DRYRUN $MAKEDIRS "$SRC_PATH" $BUCKET:$PREFIX"$DEST_PATH"
	ERROR=$?
	ishfs "$SRC_PATH" && dot_clean "$SRC_PATH"
else
	writeout "  Source:      $BUCKET:$PREFIX$SRC_PATH"
	writeout "  Destination: $DEST_PATH"
	writeout ""
	writeout "  Starting s3sync... "
	ruby $S3SYNC -r $DELETE $NOMD5 $VERBOSE $DRYRUN $MAKEDIRS $BUCKET:$PREFIX"$SRC_PATH" "$DEST_PATH"
	ERROR=$?
	ishfs "$DEST_PATH" && dot_clean "$DEST_PATH"
fi

writeout "  ... done."
exit $ERROR
